// -*- C++ -*-

#ifndef CMB_BPGRAPH_HPP
#define CMB_BPGRAPH_HPP

#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <iostream>

namespace cmb
{
  typedef std::vector<int> Seq;

  enum GraphType
    {
      LINEAR,
      CYCLIC
    };

  template <typename Iter>
  struct Range
  {
    typedef typename std::iterator_traits<Iter>::value_type value_type;
    typedef std::pair<Iter, Iter> Pair;
    Range(const Pair& x) : m_pair(x) {}
    operator bool() const { return m_pair.first != m_pair.second; }
    Iter& operator ++() { assert(*this); return ++m_pair.first; } // prefix
    Iter operator ++(int) {assert(*this); return m_pair.first++; }// postfix
    value_type operator *() { return *m_pair.first; }

    Pair m_pair;
  };

  namespace bp
  {
    struct Vert
    {
      Vert();
      unsigned label; // gene label
      bool positive;  // true => canonically on pos (right) side of gene
      unsigned cycle; // cycle (or component in the graph sense)
    };

    struct Edge
    {
      Edge();
      bool reality; // is a source edge?
      unsigned cycle;
    };

    typedef boost::adjacency_list<boost::vecS, 
                                  boost::vecS, 
                                  boost::undirectedS,
                                  Vert,
                                  Edge> Graph;

    Graph make(const Seq& x, GraphType t = LINEAR);
    void drawSVG(std::ostream& out, Graph& g);

    void drawSVG(std::ostream& out, const Seq& s, GraphType t = LINEAR);
  }

  namespace cg
  {
    struct Vert
    {
      Vert();
      unsigned min;       // bp::VertDesc range
      unsigned max;       // overlapping ranges cross
      bool directed;      // true => part of directed component
      unsigned component; // component index/color
    };

    struct Edge
    {
      Edge();
      unsigned component;
    };

    typedef boost::adjacency_list<boost::vecS, 
                                  boost::vecS, 
                                  boost::undirectedS,
                                  Vert,
                                  Edge> Graph;

    Graph make(bp::Graph& bp);

    struct Component
    {
      Component(unsigned e, bool d) : exemplar(e), directed(d) {}
      unsigned exemplar;
      bool directed;
    };
    
    typedef std::vector<Component> Components;

    Components components(Graph& cg, const bp::Graph& bp);
  }
}

#endif
