//This file provides the method to pick the best insertion locations given
//the information in the possible
//clumping matrix.

#include "ins-selection.h"

// This is a brutally ugly hack that avoids the bug in Microsoft Visual C++ 6.0
//  that leads to compiler error C2955. This will only work as long as STL sets
//  aren't used in this file. If that occurs and you want Windows portability,
//  switch to a newer MS C++ compiler
// http://support.microsoft.com/default.aspx?scid=kb;EN-US;Q275674
#ifdef _MSC_VER
#if (_MSC_VER == 1200)
#define set Set
#endif //(_MSC_VER == 1200)
#endif //_MSC_VER

//here we have a bunch of helper functions to do subset cover
//our sets will be arrays whose first element is the cardinality of the set.
int** getSubSets(int** insMatrix, int rows, int columns);
int* makeSet(int* elems, int card);
int member(int elem, int* set); //returns the index that elem is at 0 if not present
void removeElem(int* set, int elem);
void setDiff(int* s1, int* s2); //return s1 - s2 mutating s1
int compareSetCard(int* s1, int* s2); //return 1 iff |s1| >= |s2|
int isEmpty(int* set);
int* getMaxSet(int** sets, int numSets, int* loc);
void stripVals(int* set, int** allSets, int numSets); //for all s in allSets do s - set

						      /*
int main()
{
  int** mat;
  int* temp;
  int numIns, i;

  //int r0[] = {0, 1, 1, 0, 0, 1};
  //int r1[] = {1, 0, 0, 1, 0, 0};
  //int r2[] = {0, 0, 0, 0, 1, 0};
  //int r3[] = {0, 1, 0, 0, 1, 1};
  //int r4[] = {0, 1, 1, 0, 0, 1};
  //int r5[] = {0, 1, 0, 0, 1, 1};
  //int r6[] = {1, 0, 0, 1, 0, 0};

  int r0[] = {0, 0, 0, 0};
  int r1[] = {1, 0, 0, 0};
  int r2[] = {0, 1, 1, 0};
  int r3[] = {0, 0, 0, 0};
  int r4[] = {0, 1, 1, 0};
  int r5[] = {0, 0, 0, 1};
  int r6[] = {1, 0, 0, 0};

  mat = (int**) calloc(7, sizeof(int*));
  mat[0] = r0;
  mat[1] = r1;
  mat[2] = r2;
  mat[3] = r3;
  mat[4] = r4;
  mat[5] = r5;
  mat[6] = r6;
  
  temp = insLocs(mat, 7, 6, &numIns);

  for(i = 0; i < numIns; ++i)
    fprintf(stderr, "%i ", temp[i]);
  fprintf(stderr, "\n");

  return 0;
}
*/


void insLocs(int** insMatrix, int rows, int columns, int* numIns)
{
  int** subSets;
  int* set;
  int* temp; 
  int i, subSetSize, resSize, loc;

  subSetSize = rows;
  resSize = 0;

  resSize = 0;

  set = (int*) calloc(columns + 1, sizeof(int));
  set[0] = columns;
  for(i = 1; i <= columns; ++i)
    set[i] = i - 1;

/*
  for(i = 1; i <= set[0]; ++i)
    fprintf(stderr, "%i ", set[i]);
  fprintf(stderr, "\n");
*/

  subSets = getSubSets(insMatrix, rows, columns);

  while(! isEmpty(set))
    {
      temp = getMaxSet(subSets, subSetSize, &loc);
#ifdef SORTING_SEQUENCE
      if (globalSortSequence) {
	globalSortSequence->addInsSubset(temp);
      }
#endif // SORTING_SEQUENCE

      /*
      fprintf(stderr, "picked: \n");
      for(i = 1; i <= temp[0]; ++i)
	fprintf(stderr, "%i ",  temp[i]);
      fprintf(stderr, "\n");
      */

      setDiff(set, temp);
      stripVals(temp, subSets, subSetSize);
      ++resSize;
    }

  *numIns = resSize;
#ifdef SORTING_SEQUENCE
  if (globalSortSequence) {
    //globalSortSequence->printInsFinalLocs();
    //globalSortSequence->printInsSubsets();
  }
#endif // SORTING_SEQUENCE
  for (i = 0; i < rows; ++i)
    free(subSets[i]);
  free(subSets);
  free(set);
  return;
}

int** getSubSets(int** insMatrix, int rows, int columns)
{
  int i, j, setLen;
  int* set;
  int** res;

  res = (int**) calloc(rows + 1, sizeof(int*));

  for(i = 0; i < rows; ++i)
    {
      setLen = 0;
      set = (int*) calloc(columns + 2, sizeof(int));

      for(j = 0; j < columns; ++j)
	{
	  if(insMatrix[i][j] > 0)
	    {
	      set[setLen + 1] = j;
	      ++setLen;
	    }
	  else
	    ;
	}

      set[0] = setLen;

      /*
      for(j = 1; j <= set[0]; ++j)
	fprintf(stderr, "%i ", set[j]);
      fprintf(stderr, "done\n");
      */

      res[i] = set;
    }

  return res;
}

int* makeSet(int* elems, int card)
{
  int i;
  int* res;
  
  res = (int*) calloc(card + 1, sizeof(int));
  
  res[0] = card;
  for(i = 0; i < card; ++i)
    res[i + 1] = elems[i];

  return res;
}

int member(int elem, int* set)
{
  int i, found;
  found = 0;
  for(i = 1; i <= set[0]; ++i)
    {
      if(set[i] == elem)
	found = i;
      else;
    }
  
  return found;
}

void removeElem(int* set, int elem)
{
  int loc;
  
  loc = member(elem, set);

  
  //for(i = 1; i <= set[0]; ++i)
  //  fprintf(stderr, "%i ", set[i]);
  //fprintf(stderr, "\n");

  //fprintf(stderr, "here %i %i %i \n", elem, set[loc], loc);
  

  if(loc == 0)
    ;
  else if(loc == set[0])
    set[0] = set[0] - 1; //just drop it off the end
  else
    {
      set[loc] = set[set[0]]; //swap the elem and the last elem locations
      set[0] = set[0] - 1; //drop the last elem off the end
    }

  return;
}

void setDiff(int* s1, int* s2) //return s1 - s2 mutating s1
{
  int i;
  
  if(s1 == s2)
    s1[0] = 0;
  else
    {
      for(i = 1; i <= s2[0]; ++i)
	removeElem(s1, s2[i]);
    }

  return;
}

int compareSetCard(int* s1, int* s2) { return (s1[0] >= s2[0]); }

int isEmpty(int* set) { return (set[0] == 0); }

int* getMaxSet(int** sets, int numSets, int* loc)
  {
  int i, maxSizeIndex;

  maxSizeIndex = 0;

  for(i = 0; i < numSets; ++i)
    {
    /*
    cerr << " car:" << sets[i][0] << endl;
    cerr << " | " << sets[maxSizeIndex] << endl;
    */
    if(compareSetCard(sets[i], sets[maxSizeIndex]))
      maxSizeIndex = i;
    }

  *loc = maxSizeIndex;
  return sets[maxSizeIndex];
  }

void stripVals(int* set, int** allSets, int numSets)
{
  int i;
  for(i = 0; i < numSets; ++i)
    {
      if(set == allSets[i])
	;
      else
	setDiff(allSets[i], set);
    }

  setDiff(set, set);

  return;
}
