/* Krister Swenson, Mark Marron                               Spring 2003

The functions used to find the minimum edit distance from the identity.
*/

#ifndef MED
#define MED


#ifdef _MSC_VER
#pragma warning (disable : 4503) // Turn off dumb Visual C++ warning
#endif

#include <cstdlib>
#include <stack>
#include <vector>
#include <ext/hash_map>
#include "../multi/SortSeq.h"	// globalSortSequence
#include "Ograph.h"
#include "Pair.h"
#include "gencov.h"
#include "ins-selection.h"





/* Return a Minimum Edit Distance estimate from the identity permutation to the
given sequence -targ-.  The operations permitted are reversals, insertions, and
deletions. Copies should be handled outside of this function.
  -targlen- is the length of -targ-.
  -largest- all values above this in -targ- are inserted elements.
*/
int getMED(int* targ, int targlen, int largest, int &inv, int &del, int &ins);
int getMED(int* targ, int targlen, int largest);


  //Fill the insMatrix array according to the extra values from the sequence.
void fillInsMatrix(int** insMatrix, int seq[], int seqlen, int numpos,
               int numins, int srclen);
  //Free the memory from insMatrix.
void killInsMatrix(int** insMatrix, int numpos);
  //Fill insMatrix with the possible insertions through pi.
void expandInsMatrix(int** insMatrix, std::vector<Pair> &pi, int stepCount,
                    int numpos, int numins);
  //Confirm that there is at least one 1 per column.
void confirmInsMatrix(int** insMatrix, int numpos, int numins);
  //Find the clump sizes of insertions. (Obsolete)
//void findClumps(int** insMatrix, int numpos, int numins);
  //Find the deletions.  Rename the sequence to be contiguous.
void findDels(std::stack<Pair> &dels, int seq[], int seqsize, int srcsize);
  //Return the number of values that were inserted from a source permutation
  // with -srclen- elements to give -seq-.
int getNumInserts(int seq[], int origlen, int srclen);
  //Return the maximum element.
int getMax(int seq[], int seqsize);

  //Return true if val is a member of a.
bool isMember(int val, int a[], int len);

  //Print missing element in the sequence if they exist.
void confirmRenaming(int s[], int seqlen);

  //Print an error and die if there are duplicates in the given sequence.
void dupCheck(const int* seq, const int len);


void hasElement(int elem, int s[], int seqlen);
void print2D(int **a, int, int);
void printArray(const int a[], int seqsize);
void printArray(const bool a[], int seqsize);
int* readInSeq(int &len, int &numCopies, int &srclen);

#endif
