//Krister Swenson, Mark Marron                               Spring 2003
//
// Provides the interface to the -multi- code.
//

#include "multiMED.h"
// #define RAND_SEEDED_EXTERNALLY


/* Return an approximate Minimum Edit Distance from -s1- to -s2- under 
insertions, deletions, copies, and inversions.
  - To get the sorting sequence set the SORTING_SEQUENCE constant in SortSeq.h .
  - To get the El-Mabrouk distance set the MABROUK constant in multiMED.h .
*/
int multiMED(int* s1in, int ls1, int* s2in, int ls2,
             int &inv, int &del, int &ins)
  {
  int s1[ls1], s2[ls2];
  for(int i=0; i<ls1; i++)
    s1[i] = s1in[i];
  for(int i=0; i<ls2; i++)
    s2[i] = s2in[i];
#ifdef SORTING_SEQUENCE
  // Save info on the original genome sequence
  if (globalSortSequence) {
    globalSortSequence->clear();
    globalSortSequence->setGenome1(s1, ls1);
    globalSortSequence->setGenome2(s2, ls2);
  }
#endif //SORTING_SEQUENCE

  //output("\ns1", s1, ls1);
  //output("s2", s2, ls2);

#ifndef RAND_SEEDED_EXTERNALLY
  srand(time(0));
#endif // RAND_SEEDED_EXTERNALLY
#ifndef MABROUK
  renameMulti(s1, ls1, s2, ls2);    //Rename all the copies.
#else
  removeDups(s1, ls1, s2, ls2);     //Just remove all duplicates.
#endif // MABROUK

  //output("\ns1", s1, ls1);
  //output("s2", s2, ls2);

  makeIdentity(s1, ls1, s2, ls2);   //Rename so that -s1- is the identity.

#ifdef SORTING_SEQUENCE
  // Save info on the renamed genome 2 sequence
  if (globalSortSequence) {
    globalSortSequence->setRenamedG2(s2, ls2);
  }
#endif //SORTING_SEQUENCE

  /*
  output("\ns1", s1, ls1);
  output("s2", s2, ls2);
  */

  return getMED(s2, ls2, ls1, inv, del, ins); //Get the minimum edit sequence.
  }


int multiMED(int* s1, int ls1, int* s2, int ls2) {
  int inv, del, ins;
  return multiMED(s1, ls1, s2, ls2, inv, del, ins);
}


//Same as multiMED, but make doubly sure not to calculate a sorting sequence
int multiMED_noSort(int* s1, int ls1, int* s2, int ls2, int &inv, int &del,
		    int &ins) {
# ifdef SORTING_SEQUENCE
  if (globalSortSequence) {
    SortSeq* tempSortSeq = globalSortSequence;
    globalSortSequence = NULL;
    int numOps = multiMED(s1, ls1, s2, ls2, inv, del, ins);
    globalSortSequence = tempSortSeq;
    tempSortSeq = NULL;
    return numOps;
  }
  else
# endif //SORTING_SEQUENCE
    return multiMED(s1, ls1, s2, ls2, inv, del, ins);
}


int multiMED_noSort(int* s1, int ls1, int* s2, int ls2) {
  int inv, del, ins;
  return multiMED_noSort(s1, ls1, s2, ls2, inv, del, ins);
}


/* Output a sequence to stdout.
*/
void output(const char* title, int* seq, int len)
  {
  fprintf(stderr, "%s:\n", title);
  for(int i = 0; i < len; ++i)
    //if(abs(seq[i]) > REMAPSTART)
      fprintf(stderr, "[%i]%i\t", i, seq[i]);
    //printf("%i ", seq[i]);
  fprintf(stderr, "\n");
  }
