/* Krister Swenson                                                 Winter 2007-8

Excercise the noninterfering computations.
*/

#include <iostream>
#include <vector>
#include <algorithm>
#include "../graph/Ograph.h"

#define REPS 1000
#define PERMLEN 30

using namespace std; 



bool isCommuting(const list<Pair> &set)
  {
  for(list<Pair>::const_iterator i = set.begin(); i != set.end(); i++)
    {
    list<Pair>::const_iterator j = i;
    while(++j != set.end())
      if(i->overlaps(*j))     //If the two intervals overlap then sort the 
        {                     //intervals and print message:
        vector<Pair> setv;
        std::copy(set.begin (), set.end (), back_inserter(setv));
        sort(setv.begin(), setv.end());
        cerr << "sorted NMIS:" << setv << endl;
        cerr << "offending pairs:" << *i << *j << endl;

        return false;
        }
    }

  return true;
  }


int cgsseen=1;      //Count the number of non-circle graphs encountered.
int counter=1;      //Count the number of trials so far.
int trials=1;       //Count the total number of trials so far.
float total=0;      //The total of all |MIS|/d() ratios so far.
void tryPerm(const vector<int> &perm, const Ograph &g)
  {
  int dist = g.getDistNH();

  cerr << "--------------" << endl;
  cerr << "trying[" << counter++ << "-" << cgsseen << "]:" << perm << endl;
  cerr << "\tCMIS:" << g.getCommutingMIS() << endl;

  list<Pair> nmis = g.getNonintMIS();
  cerr << "\tNMIS:" << nmis << endl;

                                 //Sanity check:
  assert(g.isNonint(nmis));          //Make sure they sort.
  assert(g.isCommuting(nmis));       //Doubl check they sort (and commute).

  if(dist > 0)                   //Compute size ratios:
    {
    float ratio = (float)nmis.size()/(float)dist;
    total += ratio;
    float avgratio = total/(float)trials++;
    cerr << "\tratio:" << nmis.size() << "/" << dist << "=" << ratio
         << "  average:" << avgratio << endl;
    }
  }

void checkCG(const vector<int> &perm, const Ograph &g)
  {
  if(!g.isCG())                      //test whether it has a chord model.
    {
    cerr << "Not a circle graph!: " << perm << endl;
    //int sum=0, psum=0;
    //for(unsigned int i=0; i<perm.size(); i++)
      //{ sum += i+1; psum += perm[i]; }
    //if(sum != psum)                  //If it's not a positive permutation.
    cgsseen++;
    //exit(1);
    }
  }






int main()
  {
  vector<int> perm;


/*
  //int tmp[7] = {6,-2,5,1,-4,-7,3};
  //int tmp[7] = {-3,-6,2,7,5,-1,4};
  //int tmp[6] = {-6,-4,-2,1,-3,-5};
  //int tmp[15] = {12,-8,-7,9,11,6,10,13,1,4,-2,15,3,14,-5};
  //int tmp[5] = {4,2,5,3,-1};
  int tmp[30] = {24,26,17,22,14,-10,7,2,-15,-29,-27,18,12,-21,6,-4,23,25,-5,-1,11,19,-13,8,20,30,-28,3,16,-9};
  Ograph g(tmp,30);
  tryPerm(perm, g);
*/

  for(int i=1; i <= PERMLEN; i++)        //Create the identity:
    perm.push_back(i);

  cerr << "Permutation Length:" << perm.size() << endl;
  for(int i=0; i < REPS; i++)
    {
    int tmp[3] = {1,2,3};
    Ograph g(tmp,3);
    Ograph h(tmp,3);
    vector<Pair> dummy;
    do
      {
      dummy.clear();
      for(unsigned int j=0; j < perm.size(); j++)
        {                                //Randomize the permutation:
        int k = random()%perm.size();    //Swap perm[j] with a random element.
        int temp = perm[k];
        perm[k] = perm[j];
        perm[j] = temp;
        }
      for(unsigned int j=0; j < perm.size(); j++)
        if(random()%2)                   //Assign signs randomly:
          perm[j] *= -1;

      g = Ograph(perm);                  //Create the HP graph and
      h = g;                             //Do this because resolve() mutates h.
      }                                  //Go until we have no hurdles.
    while(h.getDistNH() != h.resolve(dummy));

    tryPerm(perm, g);
    //checkCG(perm, g);
    }


  cerr << "done." << endl;
  return(0);
  }
