/* Krister Swenson                                                  Spring 2008

Build random graphs and print the ratio of forward-facing reality edges for
the biggest cycles.
They should be half-half.
*/

#include "../seqtools/seqfuncs.h"
#include "../tools/tools.h"
#include "../graph/Ograph.h"
#include <vector>

#define SIZE 750
#define REPS 1000

using namespace std; 


void testTwoThirds();
void testAllSizes();

int main()
  {
  cout << "size:" << SIZE << " reps:" << REPS << endl;
  //testAllSizes();
  srandom((unsigned int)time(NULL));    //Seed the random number generator.
  testTwoThirds();

  return(0);
  }


void testTwoThirds()
  {
  int length = (2*SIZE)/3;
  cout << "length of those checked:" << length << endl;
  vector<float> largestlist;
  for(int i=0; i<REPS;)
    {
    Ograph g(getRandSPerm(SIZE));
    if(g.getLargestCycleSize() == length)
      {
      float b = g.getOrientationBalanceForLargest()*length;
  
      largestlist.push_back(b);
      cout << b << endl
           << "\tLARGEST mean:" << getMean(largestlist)
           << " dev:" << getStandardDev(largestlist) << endl;
      i++;
      }
    }
  }



void testAllSizes()
  {
  vector< vector<float> > results;
  vector<float> largestlist;
  vector<float> eachlist;
  for(int i=0; i<REPS; i++)
    {
    Ograph g(getRandSPerm(SIZE));
    vector<float> b = g.getOrientationBalance();

    largestlist.push_back(b[0]);
    eachlist.push_back(getMean(b));
    cout << b << endl
         << "\tLARGEST mean:" << getMean(largestlist)
         << " dev:" << getStandardDev(largestlist) << endl
         << "\tTOTAL mean:" << getMean(eachlist) << endl;
    results.push_back(b);
    }

  vector<float> secondlargest;
  vector<float> thirdlargest;
  vector<float> fourthlargest;
  vector<float> fifthlargest;
  for(vector< vector<float> >::iterator i=results.begin(); i!=results.end();i++)
    {
    int size = i->size();
    if(size > 1)
      secondlargest.push_back((*i)[1]);
    if(size > 2)
      thirdlargest.push_back((*i)[2]);
    if(size > 3)
      fourthlargest.push_back((*i)[3]);
    if(size > 4)
      fifthlargest.push_back((*i)[4]);
    }
  
  cout << "LARGEST mean:" << getMean(largestlist) << endl;
  cout << "         dev:" << getStandardDev(largestlist) << endl;
  cout << "SECOND  mean:" << getMean(secondlargest) << endl;
  cout << "         dev:" << getStandardDev(secondlargest) << endl;
  cout << "THIRD   mean:" << getMean(thirdlargest) << endl;
  cout << "         dev:" << getStandardDev(thirdlargest) << endl;
  cout << "FOURTH  mean:" << getMean(fourthlargest) << endl;
  cout << "         dev:" << getStandardDev(fourthlargest) << endl;
  cout << "FIFTH   mean:" << getMean(fifthlargest) << endl;
  cout << "         dev:" << getStandardDev(fifthlargest) << endl;
  }
