/* Krister Swenson                                                 Winter 2007-8

A vertex that knows it's neighbors and has an integer and string label, and
can be marked as visited.
(the string label is superficial -- the equality operator only checks the
 integer label)
*/
#ifndef VERTEX_H
#define VERTEX_H

#include <iostream>
#include <assert.h>
#include <list>
#include <string>
#include <sstream>
#include <ext/hash_map>


class Vertex;

typedef __gnu_cxx::hash_map<int, Vertex*> vpmap;
std::ostream& operator<<(std::ostream& output, const vpmap& x);


class Vertex
  {
  friend std::ostream& operator<<(std::ostream& output, const Vertex& v);

  public:
    Vertex();
      //Create a vertex with label -l-.
    Vertex(int l);
      //Create a vertex with label -l- and name -slabel-.
      // (the name is not used for equality, only the label).
    Vertex(int l, std::string slabel);
    Vertex(const Vertex &v);
    Vertex& operator=(const Vertex &v);
    ~Vertex();
    bool operator!=(const Vertex &v) const;
    bool operator==(const Vertex &v) const;
    std::string getName() const;
    int getLabel() const;
    void mark();
    void unmark();
      //Unmark all the neighbors of this vertex.
    //void unmarkNeighbors();
      //Return true if none of this vertex's neighbors are marked.
    bool noMarkedNeighbor() const;
    bool getMark() const;
      //Return the begin iterator to visit all neighbors with.
    vpmap::const_iterator nBeg() const;
      //Return the end iterator to visit all neighbors with.
    vpmap::const_iterator nEnd() const;
      //Return true if the given vertex is a neighbor of this one.
    bool hasNeighbor(const Vertex *v) const;
    bool hasNeighbor(int l) const;
      //Return true if a vertex from -l- is a neighbor of this one.
    bool hasNeighbor(const std::list<int> &l) const;
    int getDegree() const;
      //Make -v- a neighbor of this vertex if it already isn't.
    void addNeighbor(Vertex *v);
      //Remove -v- as a neighbor of this vertex if it already isn't.
    void remNeighbor(Vertex *v);

  private:
    void copy(const Vertex &v);

    int label;          //The vertex label.
    std::string name;   //The string label for this vertex.
    vpmap edgemap;      //Map of incident edges (label -> vertex pointer).
    int degree;         //The degree of the vertex.
    bool themark;
  };



#endif
