/* MaxClique - for finding maximal cliques and independent sets of graphs
 * Written by Kevin O'Neill
 * Based on algorithm of Tsukiyama, Ide, Ariyoshi, and Shirakawa
 * Latest version: December 18, 2003
 *
 * Copyright (C) 2003  Kevin O'Neill
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * The author can be contacted at oneill@cs.cornell.edu, or by mail at
 *   Computer Science Department
 *   4130 Upson Hall
 *   Cornell University
 *   Ithaca, NY 14853-7501
 *
 * Modified by krister to be linked to by other programs and compiled with g++.
 */
#ifndef MAXCLIQUE_H
#define MAXCLIQUE_H

/**
 * Standard includes
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <vector>
#include <list>
#include <iostream>
#include "tools.h"


/**
 * Internal constants
 */
#define MAX_MATRIX_SIZE 500
#define CLIQUE_MODE 0
#define IS_MODE 1
#define TRUE 1
#define FALSE 0
#define PROGRAM_NAME "maxclique"

using namespace std;

/**
 * Forward declarations
 */
  //Return a vector of all maximal independent sets (clique if mode == 0).
vector< list<int> > getAllMaximal(int size, vector< vector<int> > adjmat, int mode=1);
  //Get the set from the current state of -IS-.
list<int> getSet();
  //Get all of the maximal sets recursively.
void backtrack(int i, vector< list<int> > &retval);

void initialize_adjacency_lists_and_stuff();
void print_matrix();
void print_adjacency_lists();
void output_IS();
char* load_matrix_with_ints_and_no_diagonal(int argc,char *argv[]);

#endif
