/* Krister Swenson                                                 Winter 2007-8

Functions for taking the maximum independent set of circle graphs.
  ** this is from the paper "A New Simple Algorithm for the Maximum-Weight
     Independent Set Problem on Circle Graphs" by Valiente.
  ** It runs in O(dn) time where d is the maximum number of intervals
     at a particular index.
*/
#ifndef MISCG_H
#define MISCG_H

#include <iostream>
#include <list>
#include <vector>
#include <ext/hash_map>

using namespace std;


struct MISlist;

  //The maximum independent set when including edge e.
struct MISe
  {
  int chord;  //The chord that this edge represents.
  int weight; //The weight of the chord (is currently 1 for all chords).
  int left;   //The left enpoint of the chord.
  int right;  //The right enpoint of the chord.
  int length; //The length of the edge.
  MISlist *l; //The list of the edges(chords) that are included in this one.
  };
struct MISlist
  {
  MISlist *next;
  MISe *e;
  };
  //The subproblem: T(e) is the MIS on the subproblem induced by e.
struct Tstr
  {
  MISe *e;      //The edge associated with this subproblem.
  Tstr *nextt;  //The T value that this one depends on for it's weight.
  bool used;    //true if this subproblem was used in scoring another.
  int w;        //The weight of l.
  };


//Give in -mis- a maximum independent set of the given chord model.
// The value returned is size of the set.
int getMISCG(vector<int> &model, list<int> &mis);
int getMISCG(list<int> &model, list<int> &mis);


//Print all the MISe that are part of this interval.
void getSubMIS(MISlist *elist, list<int> &mis);

#endif
