/* Krister Swenson                                                 Winter 2010
*
* This is a word of reversals.
*/


#include "Word.h"

using namespace std;


Word::Word()
  {}

/*Create a Word from the given list.
  ** Assume that it commutes with all elements.
 */
Word::Word(const list<Element> &invs)
  {
  for(list<Element>::const_iterator element = invs.begin();
      element != invs.end(); element++)
    addElement(*element);
  }


/*Create a one element Word.
*/
Word::Word(const set<int> &e)
  { theword.insert(e); }


/*Copy constructor.
 */
Word::Word(const Word &w)
  { copy(w); }

void Word::operator=(const Word &w)
  { theword.clear(); copy(w); }


/*Copy the given word.
*/
void Word::copy(const Word &w)
  {
  for(set<Element>::const_iterator i = w.theword.begin();
      i != w.theword.end(); i++)
    {
    //set<int> tmp = set<int>(i->size());
    //std::copy(i->begin(), i->end(), tmp.begin());
    set<int> tmp;
    for(set<int>::const_iterator j = i->begin(); j != i->end(); j++)
      tmp.insert(*j);
    theword.insert(tmp);
    }
  }


/*Add the given element to the word.
  ** Assume that it commutes with all elements.
*/
void Word::addElement(const set<int> &e)
  { theword.insert(e); }

/*Remove the last element.
 */
void Word::removeLast()
  {
  if(!theword.empty())
    {
    set<Element>::iterator currentlast;   //Find the last element of the word
    string currentlaststring = "{0}";
    for(set<Element>::const_iterator e = theword.begin();
        e != theword.end(); e++)
      {
      string estring = getInvSetStr(*e);
      if(comes_before(currentlaststring, estring))
        {
        currentlaststring = estring;
        currentlast = e;
        }
      }
 
    theword.erase(currentlast);           //and erase it.
    }
  }

/*Get the last element.
 */
Element Word::getLast() const
  {
  if(!theword.empty())
    {
    set<Element>::iterator currentlast;   //Find the last element of the word:
    string currentlaststring = "{0}";
    for(set<Element>::const_iterator e = theword.begin();
        e != theword.end(); e++)
      {
      string estring = getInvSetStr(*e);
      if(comes_before(currentlaststring, estring))
        {
        currentlaststring = estring;
        currentlast = e;
        }
      }
 
    return *currentlast;                   //and return it.
    }
  }

/*Return the size of the word.
 */
int Word::getSize() const
  { return theword.size(); }


/*Return true if the given element commutes with all elements of this
  word.
*/
bool Word::commutesWith(const set<int> &e) const
  {
  for(set<Element>::const_iterator i = theword.begin(); i != theword.end(); i++)
    if(!this->commutes(*i, e))
      return false;

  return true;
  }


/* Return true if the the two elements commute.
 * (e1 \subseteq e2  or  e2 \subseteq e1  or  intersection(e1,e2) = 0)
 */
bool Word::commutes(const set<int> &e1, const set<int> &e2)
  {
  set<int> intersect;
  set_intersection(e1.begin(), e1.end(), e2.begin(), e2.end(),
                   inserter(intersect, intersect.begin()));
  return (intersect.empty() || intersect.size() == e1.size() ||
          intersect.size() == e2.size());
  }


/*Remap the sets in the word according to the given permutation.
 */
void Word::remap(const vector<int> &p)
  {
  set<Element> newword;
  for(set<Element>::iterator r = theword.begin(); r != theword.end(); r++)
    {
    Element newelem;
    for(Element::iterator i = r->begin(); i != r->end(); i++)
       newelem.insert(abs(p[(*i)-1]));

    newword.insert(newelem);
    }

  theword = newword;
  }


/*Return the begin iterator.
 */
set<Element>::const_iterator Word::getItBegin() const
  { return theword.begin(); }
/*Return the end iterator.
 */
set<Element>::const_iterator Word::getItEnd() const
  { return theword.end(); }
/*Return the reverse begin iterator.
 */
set<Element>::const_reverse_iterator Word::getRevItBegin() const
  { return theword.rbegin(); }
/*Return the reverse end iterator.
 */
set<Element>::const_reverse_iterator Word::getRevItEnd() const
  { return theword.rend(); }



inline bool char_greater(const char c1, const char c2)
  {
  if(c1 == c2)
    return false;
  if(c1 == '}')
    return false;
  if(c2 == '}')
    return true;

  return c1 > c2;
  }
inline bool Word::comes_before(const string &s1, const string &s2)
  {
  if(s1.size() > s2.size())        //If s1 is longer
    {
    for(unsigned i=0; i<s2.size(); i++)    //then check each character:
      {
//cerr << s1[i] << " > " << s2[i] << endl;
      if(char_greater(s1[i], s2[i]))       //If s1 is larger
        return false;                      //then return.
      else if(char_greater(s2[i], s1[i]))  //If s2 is larger
        return true;                       //then return.
      }
    return false;                          //s2 is a prefix of s1.
    }
  else                                     //Otherwise, s2 is longer.
    {
    for(unsigned i=0; i<s1.size(); i++)    //Check each character:
      {
//cerr << s1[i] << " > " << s2[i] << endl;
      if(char_greater(s1[i], s2[i]))       //If s1 is larger
        return false;                      //then return.
      else if(char_greater(s2[i], s1[i]))  //If s2 is larger
        return true;                       //then return.
      }
    return true;                           //s1 is a prefix of s2.
    }
  }
ostream& operator<<(ostream& output, const Word& w)
  {
  vector<string> word;
  for(set<Element>::const_iterator e = w.theword.begin();
      e != w.theword.end(); e++)
    word.push_back(getInvSetStr(*e));

  sort(word.begin(), word.end(), Word::comes_before);
  for(vector<string>::const_iterator i=word.begin(); i!=word.end(); i++)
    output << *i;
  return output;
  }
