/* Krister Swenson                                                 Winter 2010
 *
 * This is a word.
*/
#ifndef WORD_H
#define WORD_H

#include <iostream>
#include <set>
#include <string>
#include <algorithm>
#include "../tools.h"



/*___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___
Type definitions:                                                             */

typedef std::set<int> Element;


/*___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___/\___
Trace definition:                                                             */


class Word
  {
  friend ostream& operator<<(ostream& output, const Word& w);
  
  public:
    Word();
      //Create a Word from the given list.
      // ** Assume that it commutes with all elements.
    Word(const std::list<Element> &invs);
      //Create a one element Word.
    Word(const set<int> &e);
    Word(const Word &w);
      //Copy the given word.
    void copy(const Word &w);
    void operator=(const Word &w);
      //Add the given element to the word.
      //  ** Assume that it commutes with all elements.
    void addElement(const std::set<int> &e);
      //Remove the last element.
    void removeLast();
      //Get the last element.
    Element getLast() const;
      //Return true if the given element commutes with all elements of this
      //word.
    bool commutesWith(const std::set<int> &e) const;

      //Remap the sets in the word according to the given permutation.
    void remap(const vector<int> &p);

      //Return the size of the word.
    int getSize() const;

      //Return the begin iterator.
    std::set<Element>::const_iterator getItBegin() const;
      //Return the end iterator.
    std::set<Element>::const_iterator getItEnd() const;
      //Return the reverse begin iterator.
    std::set<Element>::const_reverse_iterator getRevItBegin() const;
      //Return the reverse end iterator.
    std::set<Element>::const_reverse_iterator getRevItEnd() const;

      //Return true if the first Element string should come before the
      // second Element string.
    static bool comes_before(const string &s1, const string &s2);

  private:
      //Return true if the the two elements commute.
      // (e1 \subseteq e2  or  e2 \subseteq e1  or  intersection(e1,e2) = 0)
    static bool commutes(const std::set<int> &e1, const std::set<int> &e2);


    std::set<Element> theword;     //The word.
  };


#endif
